<?php
/** 
*
* @package Advanced phpBB SEO mod Rewrite
* @version $Id: phpbb_seo_class.php 2007/05/26 13:48:48 dcz Exp $
* @copyright (c) 2006 dcz - www.phpbb-seo.com
* @license http://www.opensource.org/licenses/rpl.php RPL Public License 
*
*/

/**
* phpBB_SEO Class
* www.phpBB-SEO.com
* @package Advanced phpBB3 SEO mod Rewrite
*/
class phpbb_seo {
	var	$modrtype = 0;
	var	$seo_paths = array();
	var	$seo_url = array();
	var	$seo_censored = array();
	var	$seo_delim = array();
	var	$seo_ext = array();
	var	$seo_static = array();
	var	$seo_url_filter = array();
	var	$get_var = array();
	var	$path = '';
	var	$start = '';
	var	$filename = '';
	var	$file = '';
	var	$url_in = '';
	var	$url = '';
	var	$page_url = '';
	var	$seo_opt = array();
	var	$seo_cache = array();
	var	$encoding = 'utf-8';
	/**
	* constuctor
	*/
	function phpbb_seo() {
		global $phpEx, $config, $user;
		// config
		$this->encoding = 'utf-8';
		$this->modrtype =  3; // 3 = Advanced
		$this->seo_cache = array();
		$this->start = '';
		$this->seo_opt = array( 'rem_sid' => false, // 100% SID Removal for guests thus bots : true / false
			'rem_hilit' => true, // 100% highlights Removal for guests thus  bots : true / false
			'topic_per_page' => ($config['posts_per_page'] <= 0) ? 1 : $config['posts_per_page'],
			'topic_type' => array(), // do not change
			'topic_last_page' => array(), // do not change
		);
		// --> DOMAIN SETTING <-- //
		// Path Settings
		$server_protocol = ($config['server_protocol']) ? $config['server_protocol'] : (($config['cookie_secure']) ? 'https://' : 'http://');
		$server_name = trim($config['server_name'], '/') . '/';
		$server_port = (int) $config['server_port'];
		$server_port = ($server_port <> 80) ? ':' . $server_port : '';
		$script_path = trim($config['script_path'], '/');
		$script_path = (empty($script_path) ) ? '' : $script_path . '/';
		$this->seo_path['root_url'] =  $server_protocol . $server_name; 
		$this->seo_path['phpbb_url'] =  $this->seo_path['root_url'] . $script_path;
		$this->seo_path['phpbb_script'] =  $script_path;
		// URL Settings
		$this->seo_url = array( 'cat' =>  array(), 
			'forum' =>  array(), 
			'topic' =>  array(), 
			'user' => array(),
		);
		$this->seo_censored = array();
		$this->seo_delim = array('cat' => '-c', 
			'forum' => '-f', 
			'topic' => '-t', 
			'user' => '-u',
			// Rss
			'rss_forum' => '-rf',
			// Google
			'google_forum' => '-gf'
		);
		$this->seo_ext = array('cat' => '.html', 
			'forum' => '/', 
			'topic' => '.html', 
			'user' => '.html',
			'gz_ext' => '',
		);
		$this->seo_static = array('cat' => 'cat', 
			'forum' => 'forum', 
			'topic' => 'topic', 
			'post' => 'post', 
			'user' => 'member',
			'start' => '-',
			'global_announce' => 'announces/', 
			'gz_ext' => '.gz',
			'index' => '',
			'leaders' => 'the-team.html'
		);
		if ($config['default_lang'] == 'fr') {
			$this->seo_static['user'] = 'membre';
			$this->seo_static['global_announce'] = 'annonces/';
			$this->seo_static['leaders'] = 'equipe.html';
		}
		// URL Filters
		$this->phpbb_filter = array( 'forum' => array('st' => 0, 'sk' => 't', 'sd' => 'd'),
			'topic' => array('st' => 0, 'sk' => 't', 'sd' => 'a', 'hilit' => '')
		);
		// Stop files
		$this->seo_stop_files = array('posting', 'faq', 'ucp', 'search', 'swatch', 'mcp');
		// Stop vars
		$this->seo_stop_vars = array('view=', 'mark=', 'watch=');

		// reset GET var array
		$this->get_vars = array();

		return;
	}

	// --> URL rewriting functions <--
	/**
	* Prepare Titles for URL injection
	*/
	function format_url( $url, $type = 'topic' ) {
		$url = preg_replace('`\[.*\]`U','',$url);
		$url = preg_replace('`&(amp;)?#?[a-z0-9]+;`i','-',$url);
		$url = htmlentities($url, ENT_COMPAT, $this->encoding);
		$url = preg_replace( '`&([a-z]+)(acute|uml|circ|grave|ring|cedil|slash|tilde|caron|lig);`i',"\\1", $url );
		$url = preg_replace( array('`[^a-z0-9]`i','`[-]+`') , '-', $url);
		$url = trim($url, '-');
		$url = ( empty($url) ) ? $type : strtolower($url);
		return $url;
	}
	/**
	* Rewrite URLs.
	* Using eval() is here a way to allow adding of many more cases than just the
	* regular phpBB URL rewritting without slowing up the process.
	*/
	function url_rewrite($url, $is_amp = TRUE ) {
		global $phpEx, $user;
		$url = ($is_amp) ? str_replace('&amp;', '&', $url) : $url;
		$url = str_replace('?&', '?', $url);
		$this->url = $this->url_in = $url;
		if ( strpos($this->url, ".$phpEx") === FALSE || strpos($this->url, "adm/") !== FALSE || defined('ADMIN_START') ) {
			return ($is_amp) ? str_replace('&', '&amp;', $url) : $url;
		}
		if (!empty($this->seo_cache[$url])) {
			return $this->seo_cache[$url];
		}
		// Grabb params
		$parsed_url = parse_url($this->url);
		@parse_str($parsed_url['query'], $this->get_vars);

		$this->file = basename($parsed_url['path']);
		$this->path = trim(trim(dirname($parsed_url['path']), '/'),  '.');
		$this->path = (!empty($this->path)) ? $this->path . '/' : '';

		$this->filename = trim(str_replace(".$phpEx", '', $this->file));
		if ( in_array($this->filename, $this->seo_stop_files) ) {
			return ($is_amp) ? str_replace('&', '&amp;', $url) : $url;
		}
		if ( !$user->data['is_registered'] ) {
			if ($this->seo_opt['rem_sid'] ) {
				unset($this->get_vars['sid']);
			}
			if ($this->seo_opt['rem_hilit'] ) {
				unset($this->get_vars['hilit']);
			}
		}
		// Reset url
		$this->url = $this->file;
		if ( @method_exists($this, $this->filename) ) {
			$this->{$this->filename}();
			// Assamble URL
			$this->url .= @$this->query_string($this->get_vars);
			$this->url = ($is_amp) ? str_replace('&', '&amp;', $this->url) : $this->url;
			$this->seo_cache[$url] = $this->seo_path['root_url'] . $this->path . $this->url . ((!empty($parsed_url['fragment'])) ? '#' . $parsed_url['fragment'] : '');
			return  $this->seo_cache[$url];
		} else {
			return (strpos('http://', $url) !==FALSE ) ? $url : $this->seo_path['phpbb_url'] . preg_replace('`\.\.?/`', '', $url);
		}
	}
	/**
	* Set the $start var proper
	* @access private
	*/
	function seo_pagination() {
		$this->start = $this->seo_start( @$this->get_vars['start'] );
		unset($this->get_vars['start']);
	}
	/**
	* URL rewritting for viewtopic.php
	* @access private
	*/
	function viewtopic() {
		$this->filter_url($this->seo_stop_vars);
		$this->path = $this->seo_path['phpbb_script'];
		if ( !empty($this->get_vars['p']) ) {
			$this->url = $this->seo_static['post'] . $this->get_vars['p'] . $this->seo_ext['topic'];
			unset($this->get_vars['p'], $this->get_vars['f'], $this->get_vars['t']);
			return;
		}
		if ( !empty($this->get_vars['t']) && !empty($this->get_vars['f']) && !empty($this->seo_url['topic'][$this->get_vars['t']]) && !empty($this->seo_url['forum'][$this->get_vars['f']])) {
			// Filter default params
			$this->filter_get_var($this->phpbb_filter['topic']);
			$this->seo_pagination();
			$this->url = $this->seo_url['topic'][$this->get_vars['t']] . $this->seo_delim['topic'] . $this->get_vars['t'] . $this->start . $this->seo_ext['topic'];
			if (@$this->seo_opt['topic_type'][$this->get_vars['t']] == POST_GLOBAL) {
				$this->url = $this->seo_static['global_announce'] . $this->url;
			} else {
				$this->url = $this->seo_url['forum'][$this->get_vars['f']] . $this->seo_delim['forum'] . $this->get_vars['f'] . $this->seo_ext['forum'] . $this->url;
			}
			unset($this->get_vars['t'], $this->get_vars['f'],$this->get_vars['p']);
		} 
		return;
	}
	/**
	* URL rewritting for viewforum.php
	* @access private
	*/
	function viewforum() {
		$this->path = $this->seo_path['phpbb_script'];
		$this->filter_url($this->seo_stop_vars);
		if ( !empty($this->get_vars['f']) && !empty($this->seo_url['forum'][$this->get_vars['f']]) ) {
			// Filter default params
			$this->filter_get_var($this->phpbb_filter['forum']);
			$this->seo_pagination();
			$this->url = $this->seo_url['forum'][$this->get_vars['f']] . $this->seo_delim['forum'] . $this->get_vars['f'] . $this->start . $this->seo_ext['forum'];
			unset($this->get_vars['f']);
		}
		return;
	}
	/**
	* URL rewritting for profile.php
	* @access private
	*/
	function memberlist() {
		$this->path = $this->seo_path['phpbb_script'];
		if ( !empty($this->get_vars['u']) && @$this->get_vars['mode'] == 'viewprofile' ) {
			$this->url =  $this->seo_static['user'] . $this->get_vars['u'] . $this->seo_ext['user'];
			unset($this->get_vars['mode'], $this->get_vars['u']);
		} elseif (@$this->get_vars['mode'] == 'leaders') {
			$this->url =  $this->seo_static['leaders'];
			unset($this->get_vars['mode']);
		}
		return;
	}
	/**
	* URL rewritting for index.php
	* @access private
	*/
	function index() {
		$this->path = $this->seo_path['phpbb_script'];
		$this->url = $this->seo_static['index'];
		return;
	}
	/**
	* Will break if a $filter pattern is foundin $url.
	* Example $filter = array("view=", "mark=");
	* @access private
	*/
	function filter_url($filter = array()) {
		foreach ($filter as $patern ) {
			if ( strpos($this->url_in, $patern) !== FALSE ) {
				unset($this->get_vars);
				$this->url = $this->url_in;
				break;
			}
		}
		return;
	}
	/**
	* Will unset all default var stored in $filter array.
	* Example $filter = array('postdays' => 0, 'topicdays' => 0, 'postorder' => 'asc');
	* @access private
	*/
	function filter_get_var($filter = array()) {
		if ( !empty($this->get_vars) ) {
			foreach ($this->get_vars as $paramkey => $paramval) {
				if ( array_key_exists($paramkey, $filter) ) {
					if ( $filter[$paramkey] ==  $this->get_vars[$paramkey] || empty( $this->get_vars[$paramkey])) {
						unset($this->get_vars[$paramkey]);
					}
				}
			}	
		}
		return;
	}
	/**
	* Will return the remaining GET vars to take care of
	* @access private
	*/
	function query_string() {
		if(empty($this->get_vars)) {
			return '';
		}
		$params = array();
		foreach($this->get_vars as $key => $value) {
			$params[] = $key . '=' . $value;
		}
		return '?' . implode('&', $params);
	}
	/**
	* Returns usable start param
	*/
	function seo_start($start) {
		return ($start > 0 ) ? $this->seo_static['start'] . intval($start) : '';
	}
	// --> Add on Functions <--
	// --> Gen stats
	/**
	* Returns usable microtime
	* Borrowed from php.net
	* Required for the phpBB SEO Google sitemaps module
	*/
	function microtime_float() {
		return array_sum(explode(' ',microtime()));
	}
}
?>
